#include "sample-3.h"

// This file is our main file that runs the script and updates the game.
// It handles drawing and moving our character and the background.
// Everything else is handled by the script, which is defined in sample-3.c

Texture2D guy = {0};
Texture2D background = {0};
Texture2D doorClosed = {0};
Texture2D doorOpened = {0};
Vector2 guyPosition = {0};
Vector2 guyTarget = {0};
ScriptContext scriptContext = {0};

static void LoadAll() {
    guy = LoadTexture("data/guy.png");
    background = LoadTexture("data/background.png");
    doorClosed = LoadTexture("data/door_closed.png");
    doorOpened = LoadTexture("data/door_opened.png");
}

static void Initialize()
{
    guyPosition = (Vector2){30, 10};
    guyTarget = guyPosition;
    scriptContext.currentStep = 0;
    scriptContext.nextStep = -1;
    scriptContext.stepTimer = 0.0f;
}

int main(void)
{
    int screenWidth = 600, screenHeight = 240;
#ifdef PLATFORM_WEB
    // needed to adapt to the dynamic size of the canvas element in browser
    double canvasWidth, canvasHeight;
    emscripten_get_element_css_size("#sample_2", &canvasWidth, &canvasHeight);
    screenWidth = (int)canvasWidth;
    screenHeight = (int)canvasHeight;
#endif
    InitWindow(screenWidth, screenHeight, "Scripting Example");
    SetTargetFPS(30);
    
    LoadAll();
    Initialize();

    while (!WindowShouldClose())
    {
        BeginDrawing();
        ClearBackground((Color){ 82, 75, 36, 255 });
        for (int x = 0; x < GetScreenWidth(); x += background.width * 2)
        {
            for (int y = 0; y < GetScreenHeight(); y += background.height * 2)
            {
                DrawTextureEx(background, (Vector2){x, y}, 0, 2.0f, WHITE);
            }
        }

        // Move the guy towards the target position and animate him with some hopping
        float hopping = 0.0f;
        float distance = Vector2Distance(guyPosition, guyTarget);
        if (distance > 0.25f) {
            Vector2 direction = Vector2Normalize(Vector2Subtract(guyTarget, guyPosition));
            float hopFrequency = 0.25f;
            hopping = fabsf(sinf(-distance * hopFrequency)) * 7.0f;
            // speed up when did a jump
            float speed = 15.0f + fabsf(cosf(distance * hopFrequency) * (hopping)) * 25.0f;
            float moveForward = fminf(speed * GetFrameTime(), distance);
            guyPosition = Vector2Add(guyPosition, Vector2Scale(direction, moveForward));
        }

        DrawTextureEx(guy, (Vector2){ guyPosition.x, guyPosition.y - hopping }, 0, 2.0f, WHITE);

        if (scriptContext.nextStep >= 0)
        {
            scriptContext.currentStep = scriptContext.nextStep;
            scriptContext.nextStep = -1;
            scriptContext.stepTimer = 0.0f;
        }

        ScriptUpdate();

        scriptContext.stepTimer += GetFrameTime();
        
        // Allow the user to reset the script so they can see it again
        const char *resetText = "Reset";
        int textWidth = MeasureText(resetText, 20);
        DrawRectangle(GetScreenWidth() - textWidth - 15, 0, textWidth + 10, 30, RED);
        DrawText(resetText, GetScreenWidth() - textWidth - 10, 5, 20, WHITE);
        if (CheckCollisionPointRec(GetMousePosition(), (Rectangle){GetScreenWidth() - textWidth - 10, 5, textWidth, 20}) &&
            IsMouseButtonReleased(MOUSE_LEFT_BUTTON)) {
            Initialize();
        }

        // draw the current step number at the bottom left of the screen
        const char *stepText = TextFormat("Current step: %d", scriptContext.currentStep);
        textWidth = MeasureText(stepText, 20);
        DrawText(stepText, 10, GetScreenHeight() - 30, 20, WHITE);

        EndDrawing();
    }

    CloseWindow();

    return 0;
}