#include "sample-3.h"

// This file contains the various conditions and actions that can be used in the script.
// Using the currently active step, the conditions and actions are only executed,
// if the current step is within the specified range.

// This function determines, if based on the current step, the specified range is active.
// If the end step is -1, only the start step is checked.
static int IsStepActive(int stepStart, int stepEnd) 
{
    if (stepEnd < 0) return scriptContext.currentStep == stepStart;
    return scriptContext.currentStep >= stepStart && scriptContext.currentStep <= stepEnd;
}

// This function draws a speech bubble with a message at the specified position.
void ActionDrawBubble(int stepStart, int stepEnd, const char* message, int x, int y)
{
    if (!IsStepActive(stepStart, stepEnd)) {
        return;
    }
    
    int textWidth = MeasureText(message, 20);
    int textHeight = 20;
    int width = textWidth + 10;
    int height = textHeight + 10;
    x = x - width / 2;
    DrawRectangle(x, y, width, height, WHITE);
    DrawRectangleLinesEx((Rectangle){x, y, width, height}, 2, BLACK);
    DrawText(message, x + 5, y + 5, 20, BLACK);
}

// This function waits for a mouse click to continue to the next step.
void ConditionWaitForMouseClick(int stepStart, int jumpToStep) {
    if (!IsStepActive(stepStart, -1)) {
        return;
    }

    const char *message = "Click to continue ...";
    int textWidth = MeasureText(message, 20);
    char animatedMessage[64] = {0};
    int len;
    for (len = 0; message[len]; len++) animatedMessage[len] = message[len];
    int dotPos = (int)(GetTime() * 5) % 4;
    animatedMessage[len - 1] = dotPos == 2 ? ':' : '.';
    animatedMessage[len - 2] = dotPos == 1 ? ':' : '.';
    animatedMessage[len - 3] = dotPos == 0 ? ':' : '.';
    DrawText(animatedMessage, GetScreenWidth() - textWidth - 10, GetScreenHeight() - 30, 20, WHITE);
    
    if (IsMouseButtonReleased(MOUSE_LEFT_BUTTON)) {
        scriptContext.nextStep = jumpToStep;
    }
}

// This function draws a sprite at the specified position.
void ActionDrawSprite(int stepStart, int stepEnd, Texture2D sprite, int x, int y) {
    if (!IsStepActive(stepStart, stepEnd)) {
        return;
    }

    DrawTextureEx(sprite, (Vector2){x, y}, 0, 2.0f, WHITE);
}

// This function waits for the guy to reach a specific position.
void ConditionWaitForGuyPosition(int stepStart, int nextStep, int x, int y) {
    if (!IsStepActive(stepStart, -1)) {
        return;
    }

    guyTarget = (Vector2){x, y};
    if (Vector2Distance(guyPosition, guyTarget) < 1.0f) {
        scriptContext.nextStep = nextStep;
    }
}

// This function waits for a specified amount of seconds.
void ConditionWaitForSeconds(int stepStart, float seconds) {
    if (!IsStepActive(stepStart, -1)) {
        return;
    }

    if (scriptContext.stepTimer >= seconds) {
        scriptContext.nextStep = scriptContext.currentStep + 1;
    }
}
