#ifndef __SAMPLE_3_H__
#define __SAMPLE_3_H__

// This is our header file, which makes variables and functions accessible to other files.

#include <raylib.h>
#include <raymath.h>
#include <math.h>

// The ScriptContext manages the script relevant data; it could be extended to hold more data,
// dependent on the requirements of the script.
typedef struct ScriptContext {
    int currentStep;
    int nextStep;
    // The stepTimer resets every time the script advances to the next step.
    // It is useful for waiting for a specific amount of time or rendering animations.
    // It is in general a good idea to keep track of time in this script structure;
    // when every action and condition uses the script context as only source for 
    // time and data, we can easily pause the script or speed it up.
    float stepTimer;
} ScriptContext;

extern Texture2D guy;
extern Texture2D background;
extern Texture2D doorClosed;
extern Texture2D doorOpened;
extern Vector2 guyPosition;
extern Vector2 guyTarget;
extern ScriptContext scriptContext;

void ConditionWaitForMouseClick(int stepStart, int jumpToStep);
void ConditionWaitForGuyPosition(int stepStart, int nextStep, int x, int y);
void ConditionWaitForSeconds(int stepStart, float seconds);

void ActionDrawBubble(int stepStart, int stepEnd, const char* message, int x, int y);
void ActionDrawSprite(int stepStart, int stepEnd, Texture2D sprite, int x, int y);

void ScriptUpdate();


#ifdef PLATFORM_WEB
#include <emscripten.h>
EMSCRIPTEN_RESULT emscripten_get_element_css_size(const char * target, double * width, double * height);
#endif

#endif